import io
import os

import fitz
from loguru import logger
from PIL import Image


class Paper:
    def __init__(self, path, title="", url="", abs="", authers=[]):
        # 初始化函数，根据pdf路径初始化Paper对象
        self.url = url  # 文章链接
        self.path = path  # pdf路径
        self.section_names = []  # 段落标题
        self.section_texts = {}  # 段落内容
        self.abs = abs
        self.title_page = 0
        self.pdf = fitz.open(self.path)  # pdf文档

        self.title = self.get_title() if title == "" else title

        self.parse_pdf()
        self.authers = authers
        self.roman_num = [
            "I",
            "II",
            "III",
            "IV",
            "V",
            "VI",
            "VII",
            "VIII",
            "IIX",
            "IX",
            "X",
        ]
        self.digit_num = [str(d + 1) for d in range(10)]
        self.first_image = ""

    def parse_pdf(self):
        self.pdf = fitz.open(self.path)  # pdf文档
        self.text_list = [page.get_text() for page in self.pdf]
        self.all_text = " ".join(self.text_list)
        self.section_page_dict = self._get_all_page_index()  # 段落与页码的对应字典
        logger.trace(f"section_page_dict {self.section_page_dict}")
        self.section_text_dict = self._get_all_page()  # 段落与内容的对应字典
        self.section_text_dict.update({"title": self.title})
        self.section_text_dict.update({"paper_info": self.get_paper_info()})
        self.pdf.close()

    def get_paper_info(self):
        first_page_text = self.pdf[self.title_page].get_text()
        if "Abstract" in self.section_text_dict.keys():
            abstract_text = self.section_text_dict["Abstract"]
        else:
            abstract_text = self.abs
        first_page_text = first_page_text.replace(abstract_text, "")
        return first_page_text

    def get_image_path(self, image_path=""):
        """
        将PDF中的第一张图保存到image.png里面，存到本地目录，返回文件名称，供gitee读取
        :param filename: 图片所在路径，"C:\\Users\\Administrator\\Desktop\\nwd.pdf"
        :param image_path: 图片提取后的保存路径
        :return:
        """
        # open file
        max_size = 0
        image_list = []
        with fitz.Document(self.path) as my_pdf_file:
            # 遍历所有页面
            for page_number in range(1, len(my_pdf_file) + 1):
                # 查看独立页面
                page = my_pdf_file[page_number - 1]
                # 查看当前页所有图片
                page.get_images()
                # 遍历当前页面所有图片
                for image_number, image in enumerate(page.get_images(), start=1):
                    # 访问图片xref
                    xref_value = image[0]
                    # 提取图片信息
                    base_image = my_pdf_file.extract_image(xref_value)
                    # 访问图片
                    image_bytes = base_image["image"]
                    # 获取图片扩展名
                    ext = base_image["ext"]
                    # 加载图片
                    image = Image.open(io.BytesIO(image_bytes))
                    image_size = image.size[0] * image.size[1]
                    if image_size > max_size:
                        max_size = image_size
                    image_list.append(image)

        for image in image_list:
            image_size = image.size[0] * image.size[1]
            if image_size == max_size:
                image_name = f"image.{ext}"
                im_path = os.path.join(image_path, image_name)
                logger.info(f"image_path: {im_path}")

                max_pix = 480
                min(image.size[0], image.size[1])

                if image.size[0] > image.size[1]:
                    min_pix = int(image.size[1] * (max_pix / image.size[0]))
                    newsize = (max_pix, min_pix)
                else:
                    min_pix = int(image.size[0] * (max_pix / image.size[1]))
                    newsize = (min_pix, max_pix)
                image = image.resize(newsize)

                image.save(open(im_path, "wb"))
                return im_path, ext
        return None, None

    # 定义一个函数，根据字体的大小，识别每个章节名称，并返回一个列表
    def get_chapter_names(
        self,
    ):
        # # 打开一个pdf文件
        doc = fitz.open(self.path)  # pdf文档
        text_list = [page.get_text() for page in doc]
        all_text = ""
        for text in text_list:
            all_text += text
        # # 创建一个空列表，用于存储章节名称
        chapter_names = []
        for line in all_text.split("\n"):
            line.split(" ")
            if "." in line:
                point_split_list = line.split(".")
                space_split_list = line.split(" ")
                if 1 < len(space_split_list) < 5:
                    if 1 < len(point_split_list) < 5 and (
                        point_split_list[0] in self.roman_num
                        or point_split_list[0] in self.digit_num
                    ):
                        logger.info(f"{line=}")
                        chapter_names.append(line)
                    # 这段代码可能会有新的bug，本意是为了消除"Introduction"的问题的！
                    elif 1 < len(point_split_list) < 5:
                        logger.info(f"{line=}")
                        chapter_names.append(line)

        return chapter_names

    def get_title(self):
        doc = self.pdf  # 打开pdf文件
        max_font_size = 0  # 初始化最大字体大小为0
        max_font_sizes = [0]
        for page_index, page in enumerate(doc):  # 遍历每一页
            text = page.get_text("dict")  # 获取页面上的文本信息
            blocks = text["blocks"]  # 获取文本块列表
            for block in blocks:  # 遍历每个文本块
                if block["type"] == 0 and len(block["lines"]):  # 如果是文字类型
                    if len(block["lines"][0]["spans"]):
                        font_size = block["lines"][0]["spans"][0][
                            "size"
                        ]  # 获取第一行第一段文字的字体大小
                        max_font_sizes.append(font_size)
                        if font_size > max_font_size:  # 如果字体大小大于当前最大值
                            max_font_size = font_size  # 更新最大值
                            block["lines"][0]["spans"][0]["text"]  # 更新最大值对应的字符串
        max_font_sizes.sort()
        logger.trace(f"max_font_sizes {max_font_sizes[-10:]}")
        cur_title = ""
        for page_index, page in enumerate(doc):  # 遍历每一页
            text = page.get_text("dict")  # 获取页面上的文本信息
            blocks = text["blocks"]  # 获取文本块列表
            for block in blocks:  # 遍历每个文本块
                if block["type"] == 0 and len(block["lines"]):  # 如果是文字类型
                    if len(block["lines"][0]["spans"]):
                        cur_string = block["lines"][0]["spans"][0][
                            "text"
                        ]  # 更新最大值对应的字符串
                        block["lines"][0]["spans"][0]["flags"]  # 获取第一行第一段文字的字体特征
                        font_size = block["lines"][0]["spans"][0][
                            "size"
                        ]  # 获取第一行第一段文字的字体大小
                        # print(font_size)
                        if (
                            abs(font_size - max_font_sizes[-1]) < 0.3
                            or abs(font_size - max_font_sizes[-2]) < 0.3
                        ):
                            # print("The string is bold.", max_string, "font_size:", font_size, "font_flags:", font_flags)
                            if len(cur_string) > 4 and "arXiv" not in cur_string:
                                # print("The string is bold.", max_string, "font_size:", font_size, "font_flags:", font_flags)
                                if cur_title == "":
                                    cur_title += cur_string
                                else:
                                    cur_title += " " + cur_string
                            self.title_page = page_index
                            # break
        title = cur_title.replace("\n", " ")
        return title

    def _get_all_page_index(self):
        # 定义需要寻找的章节名称列表
        section_list = [
            "Abstract",
            "Introduction",
            "Related Work",
            "Background",
            "Introduction and Motivation",
            "Computation Function",
            "Routing Function",
            "Preliminary",
            "Problem Formulation",
            "Methods",
            "Methodology",
            "Method",
            "Approach",
            "Approaches",
            # exp
            "Materials and Methods",
            "Experiment Settings",
            "Experiment",
            "Experimental Results",
            "Evaluation",
            "Experiments",
            "Results",
            "Findings",
            "Data Analysis",
            "Discussion",
            "Results and Discussion",
            "Conclusion",
            "References",
        ]
        # 初始化一个字典来存储找到的章节和它们在文档中出现的页码
        section_page_dict = {}
        # 遍历每一页文档
        for page_index, page in enumerate(self.pdf):
            # 获取当前页面的文本内容
            cur_text = page.get_text()
            # 遍历需要寻找的章节名称列表
            for section_name in section_list:
                # 将章节名称转换成大写形式
                section_name_upper = section_name.upper()
                # 如果当前页面包含"Abstract"这个关键词
                if "Abstract" == section_name and section_name in cur_text:
                    # 将"Abstract"和它所在的页码加入字典中
                    section_page_dict[section_name] = page_index
                # 如果当前页面包含章节名称，则将章节名称和它所在的页码加入字典中
                else:
                    if section_name + "\n" in cur_text:
                        section_page_dict[section_name] = page_index
                    elif section_name_upper + "\n" in cur_text:
                        section_page_dict[section_name] = page_index
        # 返回所有找到的章节名称及它们在文档中出现的页码
        return section_page_dict

    def _get_all_page(self):
        """
        获取PDF文件中每个页面的文本信息，并将文本信息按照章节组织成字典返回。

        Returns:
            section_dict (dict): 每个章节的文本信息字典，key为章节名，value为章节文本。
        """
        text_list = []
        section_dict = {}

        # 再处理其他章节：
        text_list = [page.get_text() for page in self.pdf]
        for sec_index, sec_name in enumerate(self.section_page_dict):
            logger.trace(
                f"{sec_index=}, {sec_name=}, {self.section_page_dict[sec_name]}"
            )
            if sec_index <= 0 and self.abs:
                continue
            else:
                # 直接考虑后面的内容：
                start_page = self.section_page_dict[sec_name]
                if sec_index < len(list(self.section_page_dict.keys())) - 1:
                    end_page = self.section_page_dict[
                        list(self.section_page_dict.keys())[sec_index + 1]
                    ]
                else:
                    end_page = len(text_list)
                logger.trace(f"{start_page=}, {end_page=}")
                cur_sec_text = ""
                if end_page - start_page == 0:
                    if sec_index < len(list(self.section_page_dict.keys())) - 1:
                        next_sec = list(self.section_page_dict.keys())[sec_index + 1]
                        if text_list[start_page].find(sec_name) == -1:
                            start_i = text_list[start_page].find(sec_name.upper())
                        else:
                            start_i = text_list[start_page].find(sec_name)
                        if text_list[start_page].find(next_sec) == -1:
                            end_i = text_list[start_page].find(next_sec.upper())
                        else:
                            end_i = text_list[start_page].find(next_sec)
                        cur_sec_text += text_list[start_page][start_i:end_i]
                else:
                    for page_i in range(start_page, end_page):
                        #                         print("page_i:", page_i)
                        if page_i == start_page:
                            if text_list[start_page].find(sec_name) == -1:
                                start_i = text_list[start_page].find(sec_name.upper())
                            else:
                                start_i = text_list[start_page].find(sec_name)
                            cur_sec_text += text_list[page_i][start_i:]
                        elif page_i < end_page:
                            cur_sec_text += text_list[page_i]
                        elif page_i == end_page:
                            if sec_index < len(list(self.section_page_dict.keys())) - 1:
                                next_sec = list(self.section_page_dict.keys())[
                                    sec_index + 1
                                ]
                                if text_list[start_page].find(next_sec) == -1:
                                    end_i = text_list[start_page].find(next_sec.upper())
                                else:
                                    end_i = text_list[start_page].find(next_sec)
                                cur_sec_text += text_list[page_i][:end_i]
                section_dict[sec_name] = cur_sec_text.replace("-\n", "").replace(
                    "\n", " "
                )
        return section_dict



import datetime
from typing import Optional

import tenacity
from loguru import logger
from pydantic import BaseModel

from ..paper_with_image import Paper
from ..provider import biorxiv
from ..reader import BaseReader


class Params(BaseModel):
    date: Optional[str] = None
    days: Optional[int] = None
    server: str
    category: list[str]
    filter_keys: Optional[list[str]] = None
    max_results: int
    sort: str
    save_image: bool
    file_format: str
    language: str


class Reader(BaseReader):
    def __init__(
        self,
        category: list[str],
        filter_keys: list[str],
        root_path=".",
        sort=biorxiv.SortCriterion.SubmittedDate,
        user_name="defualt",
        args=None,
    ):
        if args is None:
            raise ValueError("args is None")

        if args.language == "en":
            language = "English"
        elif args.language == "zh":
            language = "Chinese"
        else:
            language = "English"

        super().__init__(
            filter_keys,
            root_path,
            language,
            args.file_format,
            args.save_image,
        )

        self.user_name = user_name  # 读者姓名
        self.sort = sort  # 读者选择的排序方式
        self.args = args
        self.category = category

    def get_biorxiv(self, max_results=30):
        if self.args.days is not None:
            return biorxiv.Search(
                days=self.args.days,
                server=self.args.server,
                max_results=max_results,
            )
        else:
            start_date, end_date = self.args.date.strip().split(":")
            return biorxiv.Search(
                start_date=start_date,
                end_date=end_date,
                server=self.args.server,
                max_results=max_results,
            )

    def filter_arxiv(self, max_results=30) -> list[biorxiv.Result]:
        search = self.get_biorxiv(max_results=max_results)
        results = list(search.results())

        logger.info("all search:")
        for index, result in enumerate(results):
            logger.info(f"{index=}, {result.title=}, {result.date}")

        # if self.filter_keys is empty then do not filter out
        if not self.filter_keys:
            return results

        filter_results = []

        logger.info(f"filter_keys {self.filter_keys}")
        # 确保每个关键词都能在摘要中找到，才算是目标论文
        for index, result in enumerate(results):
            abs_text = result.abstract.replace("-\n", "-").replace("\n", " ").lower()
            meet_num = 0

            for f_key in self.filter_keys:
                if f_key.lower() in abs_text:
                    meet_num += 1

            if meet_num == len(self.filter_keys):
                filter_results.append(result)

        logger.info(f"filter_results: {len(filter_results)}")
        logger.info("filter_papers:")
        for index, result in enumerate(filter_results):
            logger.info(f"{index=}, {result.title=}, {result.date}")

        return filter_results

    def download_pdf(self, filter_results):
        # 先创建文件夹
        date_str = str(datetime.datetime.now())[:13].replace(" ", "-")

        category_str = "-".join([c for c in self.category])

        path = self.root_path / "pdf_files" / f"{category_str}-{date_str}"
        path.mkdir(parents=True, exist_ok=True)

        logger.info(f"All_paper: {len(filter_results)}")
        # 开始下载：
        paper_list = []
        for _, result in enumerate(filter_results):
            try:
                title_str = self.validateTitle(result.title)
                pdf_name = title_str + ".pdf"
                self.try_download_pdf(result, path.as_posix(), pdf_name)
                paper_path = path / pdf_name

                logger.trace(f"{paper_path=}")
                paper = Paper(
                    path=paper_path,
                    url=result.entry_id,
                    title=result.title,
                    abs=result.abstract.replace("-\n", "-").replace("\n", " "),
                    authers=[str(aut) for aut in result.authors.split(",")],
                )
                paper_list.append(paper)

            except Exception as e:
                logger.warning(f"download_error: {e}")
                pass

        return paper_list

    @tenacity.retry(
        wait=tenacity.wait_exponential(multiplier=1, min=4, max=10),
        stop=tenacity.stop_after_attempt(5),
        reraise=True,
    )
    def try_download_pdf(self, result, path, pdf_name):
        result.download_pdf(path, filename=pdf_name)

    def show_info(self):
        categories = ",".join(self.category)
        logger.info(f"Categories: {categories}")
        logger.info(f"Sort: {self.sort}")
        logger.info(f"Max_results: {self.args.max_results}")
        if self.args.days is not None:
            logger.info(f"Day: {self.args.days}")
        else:
            logger.info(f"Date: {self.args.date}")


CATEGORY_LIST = [
    "animal behavior and cogtition",
    "biochemistry",
    "bioengineering",
    "bioinformatics",
    "biophysics",
    "cancer biology",
    "clinical trials",
    "developmental biology",
    "ecology",
    "epidemiology",
    "evolutionary biology",
    "genetics",
    "genomics",
    "immunology",
    "microbiology",
    "molecular biology",
    "neuroscience",
    "paleontology",
    "pathology",
    "pharmacology and toxicology",
    "physiology",
    "plant biology",
    "scientific communication and education",
    "synthetic biology",
    "systems biology",
    "zoology",
]


def add_subcommand(parser):
    name = "biorxiv"
    subparser = parser.add_parser(name, help="Fetch and Summary paper from bioarxiv")

    subparser.add_argument(
        "--category",
        type=str,
        default="bioinformatics",
        choices=CATEGORY_LIST,
        action="extend",
        nargs="+",
        metavar="",
        help="the category of user research fields (default: %(default)s)",
    )

    group = subparser.add_mutually_exclusive_group()
    group.add_argument(
        "--date",
        type=str,
        metavar="",
        help="the date of user research fields (example 2018-08-21:2018-08-28)",
    )

    group.add_argument(
        "--days",
        type=int,
        default=2,
        metavar="",
        help="the last days of arxiv papers of this query (default: %(default)s)",
    )

    subparser.add_argument(
        "--server",
        type=str,
        default="biorxiv",
        choices=["biorxiv", "medrxiv"],
        metavar="",
        help="the category of user research fields (default: %(default)s)",
    )

    subparser.add_argument(
        "--filter-keys",
        type=str,
        action="extend",
        nargs="+",
        metavar="",
        help="the filter key words, every word in the abstract must have, otherwise it will not be selected as the target paper",
    )

    subparser.add_argument(
        "--max-results",
        type=int,
        default=20,
        metavar="",
        help="the maximum number of results (default: %(default)s)",
    )

    subparser.add_argument(
        "--sort",
        type=str,
        default="Relevance",
        metavar="",
        help="another is LastUpdatedDate (default: %(default)s)",
    )

    subparser.add_argument(
        "--save-image",
        default=False,
        metavar="",
        help="save image? It takes a minute or two to save a picture! But pretty (default: %(default)s)",
    )

    subparser.add_argument(
        "--file-format",
        type=str,
        default="md",
        choices=["md", "txt", "pdf"],
        metavar="",
        help="the format of the exported file, if you save the picture, it is best to be md, if not, the txt will not be messy (default: %(default)s)",
    )

    subparser.add_argument(
        "--language",
        type=str,
        default="en",
        metavar="",
        help="The other output lauguage is English, is en (default: %(default)s)",
    )

    return name


def main(args):
    if args.sort == "Relevance":
        sort = biorxiv.SortCriterion.Relevance
    elif args.sort == "LastUpdatedDate":
        sort = biorxiv.SortCriterion.LastUpdatedDate
    else:
        sort = biorxiv.SortCriterion.Relevance

    reader = Reader(
        category=args.category,
        filter_keys=args.filter_keys,
        sort=sort,
        args=args,
    )

    reader.show_info()
    filter_results = reader.filter_arxiv(max_results=args.max_results)
    paper_list = reader.download_pdf(filter_results)
    key_words = ",".join(reader.category)
    reader.summary_with_chat(
        paper_list,
        key_words,
    )


def cli(args):
    if not isinstance(args.category, list):
        args.category = [args.category]

    parser = Params(**vars(args))
    main(parser)



__version__ = "0.1.0"
__PACKAGE_NAME__ = "chat-research"


from rich.traceback import install

from . import paper, paper_with_image, utils

install()

__all__ = ["paper", "paper_with_image", "utils"]



import datetime
import os
import re

import arxiv
import openai

# 导入所需的库
import requests
import tenacity
import tiktoken
from bs4 import BeautifulSoup
from loguru import logger
from pydantic import BaseModel

from ..paper_with_image import Paper
from ..utils import load_config, report_token_usage


class ArxivParams(BaseModel):
    query: str
    key_word: str
    page_num: int
    max_results: int
    days: int
    sort: str
    save_image: bool
    file_format: str
    language: str


# 定义Reader类
class Reader:
    # 初始化方法，设置属性
    def __init__(
        self,
        key_word,
        query,
        root_path="./",
        sort=arxiv.SortCriterion.SubmittedDate,
        user_name="defualt",
        args=None,
    ):
        self.user_name = user_name  # 读者姓名
        self.key_word = key_word  # 读者感兴趣的关键词
        self.query = query  # 读者输入的搜索查询
        self.sort = sort  # 读者选择的排序方式
        self.args = args

        if args is None:
            raise ValueError("args cannot be None")

        if args.language == "en":
            self.language = "English"
        elif args.language == "zh":
            self.language = "Chinese"
        else:
            self.language = "English"

        self.root_path = root_path
        self.config, self.chat_api_list = load_config()

        self.cur_api = 0
        self.file_format = args.file_format

        if args.save_image:
            self.gitee_key = self.config["Gitee"]["api"]
        else:
            self.gitee_key = ""

        self.max_token_num = 4096
        self.encoding = tiktoken.get_encoding("gpt2")

    # 定义一个函数，根据关键词和页码生成arxiv搜索链接
    def get_url(self, keyword, page):
        base_url = "https://arxiv.org/search/?"
        params = {
            "query": keyword,
            "searchtype": "all",  # 搜索所有字段
            "abstracts": "show",  # 显示摘要
            "order": "-announced_date_first",  # 按日期降序排序
            "size": 50,  # 每页显示50条结果
        }
        if page > 0:
            params["start"] = page * 50  # 设置起始位置

        return base_url + requests.compat.urlencode(params)

    # 定义一个函数，根据链接获取网页内容，并解析出论文标题
    def get_titles(self, url, days=1):
        titles = []
        # 创建一个空列表来存储论文链接
        links = []
        dates = []
        response = requests.get(url)
        soup = BeautifulSoup(response.text, "html.parser")
        articles = soup.find_all("li", class_="arxiv-result")  # 找到所有包含论文信息的li标签
        today = datetime.date.today()
        last_days = datetime.timedelta(days=days)
        for article in articles:
            title = article.find("p", class_="title").text  # 找到每篇论文的标题，并去掉多余的空格和换行符
            link = article.find("span").find_all("a")[0].get("href")
            date_text = article.find("p", class_="is-size-7").text
            date_text = date_text.split("\n")[0].split("Submitted ")[-1].split("; ")[0]
            date_text = datetime.datetime.strptime(date_text, "%d %B, %Y").date()
            if today - date_text <= last_days:
                titles.append(title.strip())
                links.append(link)
                dates.append(date_text)
            # print("links:", links)
        return titles, links, dates

    # 定义一个函数，根据关键词获取所有可用的论文标题，并打印出来
    def get_all_titles_from_web(self, keyword, page_num=1, days=1):
        title_list, link_list, date_list = [], [], []
        for page in range(page_num):
            url = self.get_url(keyword, page)  # 根据关键词和页码生成链接
            titles, links, dates = self.get_titles(url, days)  # 根据链接获取论文标题
            if not titles:  # 如果没有获取到任何标题，说明已经到达最后一页，退出循环
                break
            for title_index, title in enumerate(titles):  # 遍历每个标题，并打印出来
                logger.info(
                    f"{page}, {title_index}, {title}, {links[title_index]}, {dates[title_index]}"
                )
            title_list.extend(titles)
            link_list.extend(links)
            date_list.extend(dates)
        logger.info("-" * 40)
        return title_list, link_list, date_list

    def get_arxiv(self, max_results=30):
        search = arxiv.Search(
            query=self.query,
            max_results=max_results,
            sort_by=self.sort,
            sort_order=arxiv.SortOrder.Descending,
        )
        return search

    def get_arxiv_web(self, args, page_num=1, days=2):
        titles, links, dates = self.get_all_titles_from_web(
            args.query, page_num=page_num, days=days
        )
        paper_list = []
        for title_index, title in enumerate(titles):
            if title_index + 1 > args.max_results:
                break
            logger.info(
                f"{title_index}, {title}, {links[title_index]}, {dates[title_index]}"
            )
            url = links[title_index] + ".pdf"  # the link of the pdf document
            filename = self.try_download_pdf(url, title)
            paper = Paper(
                path=filename,
                url=links[title_index],
                title=title,
            )
            paper_list.append(paper)
        return paper_list

    def validateTitle(self, title):
        # 将论文的乱七八糟的路径格式修正
        rstr = r"[\/\\\:\*\?\"\<\>\|]"  # '/ \ : * ? " < > |'
        new_title = re.sub(rstr, "_", title)  # 替换为下划线
        return new_title

    def download_pdf(self, url, title):
        response = requests.get(url)  # send a GET request to the url
        date_str = str(datetime.datetime.now())[:13].replace(" ", "-")
        path = (
            self.root_path
            + "pdf_files/"
            + self.validateTitle(self.query)
            + "-"
            + date_str
        )

        try:
            os.makedirs(path)
        except Exception:
            pass
        filename = os.path.join(path, self.validateTitle(title)[:80] + ".pdf")
        with open(filename, "wb") as f:  # open a file with write and binary mode
            f.write(response.content)  # write the content of the response to the file
        return filename

    @tenacity.retry(
        wait=tenacity.wait_exponential(multiplier=1, min=4, max=10),
        stop=tenacity.stop_after_attempt(5),
        reraise=True,
    )
    def try_download_pdf(self, url, title):
        return self.download_pdf(url, title)

    def summary_with_chat(self, paper_list):
        htmls = []
        for paper_index, paper in enumerate(paper_list):
            # 第一步先用title，abs，和introduction进行总结。
            text = ""
            text += "Title:" + paper.title
            text += "Url:" + paper.url
            text += "Abstrat:" + paper.abs
            text += "Paper_info:" + paper.section_text_dict["paper_info"]
            # intro
            text += list(paper.section_text_dict.values())[0]

            try:
                chat_summary_text = self.chat_summary(text=text)
            except Exception as e:
                logger.info(f"summary_error: {e}")
                if "maximum context" in str(e):
                    current_tokens_index = (
                        str(e).find("your messages resulted in")
                        + len("your messages resulted in")
                        + 1
                    )
                    offset = int(
                        str(e)[current_tokens_index : current_tokens_index + 4]
                    )
                    summary_prompt_token = offset + 1000 + 150
                    chat_summary_text = self.chat_summary(
                        text=text, summary_prompt_token=summary_prompt_token
                    )

            htmls.append("## Paper:" + str(paper_index + 1))
            htmls.append("\n\n\n")
            chat_summary_text = ""
            if "chat_summary_text" in locals():
                htmls.append(chat_summary_text)

            # 第二步总结方法：
            # TODO，由于有些文章的方法章节名是算法名，所以简单的通过关键词来筛选，很难获取，后面需要用其他的方案去优化。
            method_key = ""
            for parse_key in paper.section_text_dict.keys():
                if "method" in parse_key.lower() or "approach" in parse_key.lower():
                    method_key = parse_key
                    break

            if method_key != "":
                text = ""
                method_text = ""
                summary_text = ""
                summary_text += "<summary>" + chat_summary_text
                # methods
                method_text += paper.section_text_dict[method_key]
                text = summary_text + "\n\n<Methods>:\n\n" + method_text
                # chat_method_text = self.chat_method(text=text)
                try:
                    chat_method_text = self.chat_method(text=text)
                except Exception as e:
                    logger.info(f"method_error: {e}")
                    if "maximum context" in str(e):
                        current_tokens_index = (
                            str(e).find("your messages resulted in")
                            + len("your messages resulted in")
                            + 1
                        )
                        offset = int(
                            str(e)[current_tokens_index : current_tokens_index + 4]
                        )
                        method_prompt_token = offset + 800 + 150
                        chat_method_text = self.chat_method(
                            text=text, method_prompt_token=method_prompt_token
                        )
                chat_method_text = ""
                if "chat_method_text" in locals():
                    htmls.append(chat_method_text)
                # htmls.append(chat_method_text)
            else:
                chat_method_text = ""
            htmls.append("\n" * 4)

            # 第三步总结全文，并打分：
            conclusion_key = ""
            for parse_key in paper.section_text_dict.keys():
                if "conclu" in parse_key.lower():
                    conclusion_key = parse_key
                    break

            text = ""
            conclusion_text = ""
            summary_text = ""
            summary_text += (
                "<summary>"
                + chat_summary_text
                + "\n <Method summary>:\n"
                + chat_method_text
            )
            if conclusion_key != "":
                # conclusion
                conclusion_text += paper.section_text_dict[conclusion_key]
                text = summary_text + "\n\n<Conclusion>:\n\n" + conclusion_text
            else:
                text = summary_text
            # chat_conclusion_text = self.chat_conclusion(text=text)
            try:
                chat_conclusion_text = self.chat_conclusion(text=text)
            except Exception as e:
                logger.info(f"conclusion_error: {e}")
                if "maximum context" in str(e):
                    current_tokens_index = (
                        str(e).find("your messages resulted in")
                        + len("your messages resulted in")
                        + 1
                    )
                    offset = int(
                        str(e)[current_tokens_index : current_tokens_index + 4]
                    )
                    conclusion_prompt_token = offset + 800 + 150
                    chat_conclusion_text = self.chat_conclusion(
                        text=text, conclusion_prompt_token=conclusion_prompt_token
                    )
            chat_conclusion_text = ""
            if "chat_conclusion_text" in locals():
                htmls.append(chat_conclusion_text)
            htmls.append("\n" * 4)

            # # 整合成一个文件，打包保存下来。
            date_str = str(datetime.datetime.now())[:13].replace(" ", "-")
            export_path = os.path.join(self.root_path, "export")
            if not os.path.exists(export_path):
                os.makedirs(export_path)
            mode = "w" if paper_index == 0 else "a"
            file_name = os.path.join(
                export_path,
                date_str
                + "-"
                + self.validateTitle(self.query)
                + "."
                + self.file_format,
            )
            self.export_to_markdown("\n".join(htmls), file_name=file_name, mode=mode)
            htmls = []

    @tenacity.retry(
        wait=tenacity.wait_exponential(multiplier=1, min=4, max=10),
        stop=tenacity.stop_after_attempt(5),
        reraise=True,
    )
    def chat_conclusion(self, text, conclusion_prompt_token=800):
        openai.api_key = self.chat_api_list[self.cur_api]
        self.cur_api += 1
        self.cur_api = (
            0 if self.cur_api >= len(self.chat_api_list) - 1 else self.cur_api
        )
        text_token = len(self.encoding.encode(text))
        clip_text_index = int(
            len(text) * (self.max_token_num - conclusion_prompt_token) / text_token
        )
        clip_text = text[:clip_text_index]

        messages = [
            {
                "role": "system",
                "content": "You are a reviewer in the field of ["
                + self.key_word
                + "] and you need to critically review this article",
            },
            # chatgpt 角色
            {
                "role": "assistant",
                "content": "This is the <summary> and <conclusion> part of an English literature, where <summary> you have already summarized, but <conclusion> part, I need your help to summarize the following questions:"
                + clip_text,
            },
            # 背景知识，可以参考OpenReview的审稿流程
            {
                "role": "user",
                "content": """
                 8. Make the following summary.Be sure to use {} answers (proper nouns need to be marked in English).
                    - (1):What is the significance of this piece of work?
                    - (2):Summarize the strengths and weaknesses of this article in three dimensions: innovation point, performance, and workload.
                    .......
                 Follow the format of the output later:
                 8. Conclusion: \n\n
                    - (1):xxx;\n
                    - (2):Innovation point: xxx; Performance: xxx; Workload: xxx;\n

                 Be sure to use {} answers (proper nouns need to be marked in English), statements as concise and academic as possible, do not repeat the content of the previous <summary>, the value of the use of the original numbers, be sure to strictly follow the format, the corresponding content output to xxx, in accordance with \n line feed, ....... means fill in according to the actual requirements, if not, you can not write.
                 """.format(
                    self.language, self.language
                ),
            },
        ]
        response = openai.ChatCompletion.create(
            model="gpt-3.5-turbo",
            # prompt需要用英语替换，少占用token。
            messages=messages,
        )
        result = ""
        for choice in response.choices:
            result += choice.message.content
        logger.info(f"conclusion_result:\n{result}")
        report_token_usage(response)

        return result

    @tenacity.retry(
        wait=tenacity.wait_exponential(multiplier=1, min=4, max=10),
        stop=tenacity.stop_after_attempt(5),
        reraise=True,
    )
    def chat_method(self, text, method_prompt_token=800):
        openai.api_key = self.chat_api_list[self.cur_api]
        self.cur_api += 1
        self.cur_api = (
            0 if self.cur_api >= len(self.chat_api_list) - 1 else self.cur_api
        )
        text_token = len(self.encoding.encode(text))
        clip_text_index = int(
            len(text) * (self.max_token_num - method_prompt_token) / text_token
        )
        clip_text = text[:clip_text_index]
        messages = [
            {
                "role": "system",
                "content": "You are a researcher in the field of ["
                + self.key_word
                + "] who is good at summarizing papers using concise statements",
            },
            # chatgpt 角色
            {
                "role": "assistant",
                "content": "This is the <summary> and <Method> part of an English document, where <summary> you have summarized, but the <Methods> part, I need your help to read and summarize the following questions."
                + clip_text,
            },
            # 背景知识
            {
                "role": "user",
                "content": """
                 7. Describe in detail the methodological idea of this article. Be sure to use {} answers (proper nouns need to be marked in English). For example, its steps are.
                    - (1):...
                    - (2):...
                    - (3):...
                    - .......
                 Follow the format of the output that follows:
                 7. Methods: \n\n
                    - (1):xxx;\n
                    - (2):xxx;\n
                    - (3):xxx;\n
                    ....... \n\n

                 Be sure to use {} answers (proper nouns need to be marked in English), statements as concise and academic as possible, do not repeat the content of the previous <summary>, the value of the use of the original numbers, be sure to strictly follow the format, the corresponding content output to xxx, in accordance with \n line feed, ....... means fill in according to the actual requirements, if not, you can not write.
                 """.format(
                    self.language, self.language
                ),
            },
        ]
        response = openai.ChatCompletion.create(
            model="gpt-3.5-turbo",
            messages=messages,
        )
        result = ""
        for choice in response.choices:
            result += choice.message.content
        logger.info("method_result:\n", result)
        report_token_usage(response)

        return result

    @tenacity.retry(
        wait=tenacity.wait_exponential(multiplier=1, min=4, max=10),
        stop=tenacity.stop_after_attempt(5),
        reraise=True,
    )
    def chat_summary(self, text, summary_prompt_token=1100):
        openai.api_key = self.chat_api_list[self.cur_api]
        self.cur_api += 1
        self.cur_api = (
            0 if self.cur_api >= len(self.chat_api_list) - 1 else self.cur_api
        )
        text_token = len(self.encoding.encode(text))
        clip_text_index = int(
            len(text) * (self.max_token_num - summary_prompt_token) / text_token
        )
        clip_text = text[:clip_text_index]
        messages = [
            {
                "role": "system",
                "content": "You are a researcher in the field of ["
                + self.key_word
                + "] who is good at summarizing papers using concise statements",
            },
            {
                "role": "assistant",
                "content": "This is the title, author, link, abstract and introduction of an English document. I need your help to read and summarize the following questions: "
                + clip_text,
            },
            {
                "role": "user",
                "content": """
                 1. Mark the title of the paper (with Chinese translation)
                 2. list all the authors' names (use English)
                 3. mark the first author's affiliation (output {} translation only)
                 4. mark the keywords of this article (use English)
                 5. link to the paper, Github code link (if available, fill in Github:None if not)
                 6. summarize according to the following four points.Be sure to use {} answers (proper nouns need to be marked in English)
                    - (1):What is the research background of this article?
                    - (2):What are the past methods? What are the problems with them? Is the approach well motivated?
                    - (3):What is the research methodology proposed in this paper?
                    - (4):On what task and what performance is achieved by the methods in this paper? Can the performance support their goals?
                 Follow the format of the output that follows:
                 1. Title: xxx\n\n
                 2. Authors: xxx\n\n
                 3. Affiliation: xxx\n\n
                 4. Keywords: xxx\n\n
                 5. Urls: xxx or xxx , xxx \n\n
                 6. Summary: \n\n
                    - (1):xxx;\n
                    - (2):xxx;\n
                    - (3):xxx;\n
                    - (4):xxx.\n\n

                 Be sure to use {} answers (proper nouns need to be marked in English), statements as concise and academic as possible, do not have too much repetitive information, numerical values using the original numbers, be sure to strictly follow the format, the corresponding content output to xxx, in accordance with \n line feed.
                 """.format(
                    self.language, self.language, self.language
                ),
            },
        ]

        response = openai.ChatCompletion.create(
            model="gpt-3.5-turbo",
            messages=messages,
        )

        result = ""
        for choice in response.choices:
            result += choice.message.content
        logger.info(f"summary_result:\n{result}")

        report_token_usage(response)

        return result

    def export_to_markdown(self, text, file_name, mode="w"):
        # 打开一个文件，以写入模式
        with open(file_name, mode, encoding="utf-8") as f:
            # 将html格式的内容写入文件
            f.write(text)

    # 定义一个方法，打印出读者信息
    def show_info(self):
        logger.info(f"Key word: {self.key_word}")
        logger.info(f"Query: {self.query}")
        logger.info(f"Sort: {self.sort}")


def add_subcommnd(parser):
    name = "arxiv"
    subparser = parser.add_parser(name, help="Fetch and summary paper from arxiv")
    subparser.add_argument(
        "--query",
        type=str,
        default="GPT-4",
        metavar="",
        help="the query string, ti: xx, au: xx, all: xx,",
    )
    subparser.add_argument(
        "--key-word",
        type=str,
        default="GPT robot",
        metavar="",
        help="the key word of user research fields",
    )

    subparser.add_argument(
        "--page-num", type=int, default=1, metavar="", help="the maximum number of page"
    )
    subparser.add_argument(
        "--max-results",
        type=int,
        default=1,
        metavar="",
        help="the maximum number of results",
    )
    subparser.add_argument(
        "--days",
        type=int,
        default=1,
        metavar="",
        help="the last days of arxiv papers of this query",
    )
    subparser.add_argument(
        "--sort", type=str, default="web", metavar="", help="another is LastUpdatedDate"
    )
    subparser.add_argument(
        "--save-image",
        default=False,
        metavar="",
        help="save image? It takes a minute or two to save a picture! But pretty",
    )

    subparser.add_argument(
        "--file-format",
        type=str,
        default="md",
        metavar="",
        help="export file format, if you want to save pictures, md is the best, if not, txt will not be messy",
    )

    subparser.add_argument(
        "--language",
        type=str,
        default="en",
        metavar="",
        help="The other output lauguage is English, is en",
    )

    return name


def main(args):
    reader1 = Reader(key_word=args.key_word, query=args.query, args=args)
    reader1.show_info()
    paper_list = reader1.get_arxiv_web(
        args=args, page_num=args.page_num, days=args.days
    )

    reader1.summary_with_chat(paper_list=paper_list)


def cli(args):
    parameters = ArxivParams(**vars(args))
    main(parameters)



import argparse
import logging
from contextlib import redirect_stdout
from io import StringIO
from pprint import pformat

from chat_arxiv import ArxivParams, chat_arxiv_main
from chat_paper import PaperParams, chat_paper_main
from chat_response import ResponseParams, chat_response_main
from chat_reviewer import ReviewerParams, chat_reviewer_main
from flask import Flask, request, url_for
from flask_cors import CORS

app = Flask(__name__)
CORS(app)


@app.route("/", methods=["GET"])
@app.route("/index", methods=["GET"])
def home():
    arxiv_url = url_for(
        "arxiv",
        _external=True,
        query="GPT-4",
        key_word="GPT robot",
        page_num=1,
        max_results=1,
        days=1,
        sort="web",
        save_image=False,
        file_format="md",
        language="zh",
    )
    paper_url = url_for(
        "paper",
        _external=True,
        pdf_path="",
        query="all: ChatGPT robot",
        key_word="reinforcement learning",
        filter_keys="ChatGPT robot",
        max_results=1,
        sort="Relevance",
        save_image=False,
        file_format="md",
        language="zh",
    )
    response_url = url_for(
        "response",
        _external=True,
        comment_path="review_comments.txt",
        file_format="txt",
        language="en",
    )
    reviewer_url = url_for(
        "reviewer",
        _external=True,
        paper_path="",
        file_format="txt",
        research_fields="computer science, artificial intelligence and reinforcement learning",
        language="en",
    )
    return f"""
    <h1>ChatPaper，Flask版本的优势</h1>
    <p>GitHub 项目地址：<a href="https://github.com/kaixindelele/ChatPaper" target="_blank">https://github.com/kaixindelele/ChatPaper</a></p>
    <p>将原始的 Python 脚本改为使用 Flask 构建的 Web 服务具有以下优点：</p>
    <ul>
        <li><strong>易用性</strong>：通过简单的 HTTP 请求，用户可以轻松访问和使用各个功能，无需在本地安装 Python 或其他依赖。</li>
        <li><strong>跨平台兼容性</strong>：作为 Web 服务，Flask 应用程序可以在任何支持 HTTP 请求的设备和平台上运行，从而提高了应用程序的可访问性。</li>
        <li><strong>可扩展性</strong>：使用 Flask 可以更轻松地扩展应用程序，以包含其他功能、中间件和 API 端点。</li>
        <li><strong>易于集成</strong>：Flask 应用程序可以与其他 Web 服务和前端框架（如 React、Vue.js 等）轻松集成，从而提供更丰富的用户体验。</li>
    </ul>
    <h1>功能描述和调用方法</h1>
    <h2>arxiv</h2>
    <p>搜索 Arxiv 上的论文。参数：query, key_word, page_num, max_results, days, sort, save_image, file_format, language</p>
    <p>示例：<a href="{arxiv_url}" target="_blank">{arxiv_url}</a></p>

    <h2>paper</h2>
    <p>搜索并分析论文。参数：pdf_path, query, key_word, filter_keys, max_results, sort, save_image, file_format, language</p>
    <p>示例：<a href="{paper_url}" target="_blank">{paper_url}</a></p>

    <h2>response</h2>
    <p>处理论文审稿评论。参数：comment_path, file_format, language</p>
    <p>示例：<a href="{response_url}" target="_blank">{response_url}</a></p>

    <h2>reviewer</h2>
    <p>查找论文审稿人。参数：paper_path, file_format, research_fields, language</p>
    <p>示例：<a href="{reviewer_url}" target="_blank">{reviewer_url}</a></p>
    """


def process_request(main_function, params_class, default_values):
    args = request.args.to_dict()
    for key, value in args.items():
        if key in default_values:
            args[key] = type(default_values[key])(value)

    params = params_class(**{**default_values, **args})
    output = StringIO()
    with redirect_stdout(output):
        main_function(args=params)

    output_str = output.getvalue()
    output_lines = [line.strip() for line in output_str.split("\n") if line.strip()]
    formatted_output_str = "\n".join(output_lines)
    return pformat(formatted_output_str)


@app.route("/arxiv", methods=["GET"])
def arxiv():
    default_values = {
        "query": "GPT-4",
        "key_word": "GPT robot",
        "page_num": 1,
        "max_results": 1,
        "days": 1,
        "sort": "web",
        "save_image": False,
        "file_format": "md",
        "language": "zh",
    }
    return process_request(chat_arxiv_main, ArxivParams, default_values)


@app.route("/paper", methods=["GET"])
def paper():
    default_values = {
        "pdf_path": "",
        "query": "all: ChatGPT robot",
        "key_word": "reinforcement learning",
        "filter_keys": "ChatGPT robot",
        "max_results": 1,
        "sort": "Relevance",
        "save_image": False,
        "file_format": "md",
        "language": "zh",
    }
    return process_request(chat_paper_main, PaperParams, default_values)


@app.route("/response", methods=["GET"])
def response():
    default_values = {
        "comment_path": "review_comments.txt",
        "file_format": "txt",
        "language": "en",
    }
    return process_request(chat_response_main, ResponseParams, default_values)


@app.route("/reviewer", methods=["GET"])
def reviewer():
    default_values = {
        "paper_path": "",
        "file_format": "txt",
        "research_fields": "computer science, artificial intelligence and reinforcement learning",
        "language": "en",
    }
    return process_request(chat_reviewer_main, ReviewerParams, default_values)


def get_log_level(args):
    if args.verbose == 2:
        return logging.INFO
    elif args.verbose > 2:
        return logging.DEBUG
    else:
        return logging.WARN


if __name__ == "__main__":
    # Initialize the main argument parser
    parser = argparse.ArgumentParser(description="Description of main script")
    parser.add_argument(
        "--debug", "-d", help="deploy debug mode", action="store_true", default=False
    )
    parser.add_argument("--verbose", "-v", action="count", default=1)

    args = parser.parse_args()

    logging.basicConfig(
        level=get_log_level(args),
        format="%(asctime)s %(message)s",
        datefmt="%m/%d/%Y %I:%M:%S %p",
    )

    if args.debug:
        app.run(debug=True, threaded=True, host="0.0.0.0", port=5000)
    else:
        app.run(debug=False, threaded=True, host="0.0.0.0", port=5000)



import shutil
import subprocess
from pathlib import Path

from loguru import logger

# https://github.com/Wandmalfarbe/pandoc-latex-template


def export(content: str, file_name: Path, keep_md: bool = False):
    if file_name.suffix == ".md" or file_name.suffix == ".txt":
        export_to_markdown(content, file_name, mode="w")
    elif file_name.suffix == ".pdf":
        export_to_pdf(content, file_name, mode="w", keep_md=keep_md)
    else:
        raise ValueError("Unsupported file format")


def combine_md(md_files: list[Path], output_file: Path):
    content = ""

    for i in md_files:
        with open(i, "r") as f:
            content += f.read()
        content += "\n"

    with open(output_file, "w") as f:
        f.write(content)


def md2pdf(md_file: Path, pdf_file: Path) -> bool:
    if shutil.which("pandoc") is None:
        raise FileNotFoundError("pandoc not found")

    cmd = [
        "pandoc",
        md_file,
        "-o",
        pdf_file,
    ]

    try:
        subprocess.check_call(cmd)
    except subprocess.CalledProcessError:
        logger.warning("failed to output pdf then fall back to md")
        return False
    else:
        return True


def export_to_markdown(text, file_name, mode="w"):
    # 使用markdown模块的convert方法，将文本转换为html格式
    # html = markdown.markdown(text)
    # 打开一个文件，以写入模式
    with open(file_name, mode, encoding="utf-8") as f:
        # 将html格式的内容写入文件
        f.write(text)


# WARN: https://github.com/Wandmalfarbe/pandoc-latex-template/issues/92
def export_to_pdf(text: str, file_name: Path, mode="w", keep_md: bool = False):
    export_to_markdown(text, file_name.with_suffix(".md"), mode)
    flag = md2pdf(file_name.with_suffix(".md"), file_name.with_suffix(".pdf"))

    if flag:
        if not keep_md:
            file_name.with_suffix(".md").unlink()



from pathlib import Path

import toml
from loguru import logger

from ..utils import CONFIG_FILE_NAME

DEFAULT_CONFIG = {
    "OpenAI": {"OPENAI_API_KEYS": ["sk-key1", "sk-key2"]},
    "Gitee": {
        "api": "your_gitee_api",
        "owner": "your_gitee_name",
        "repo": "your_repo_name",
        "path": "files_name_in_your_repo",
    },
}

MAP_NAMES = {"OPENAI_API_KEY": "OPENAI_API_KEYS"}


def create_config_file(config=DEFAULT_CONFIG, file: Path = Path(CONFIG_FILE_NAME)):
    with open(file, "w") as f:
        toml.dump(config, f)


def read_key_value_pair(pairs: list[str]) -> dict[str, str]:
    result = {}
    for pair in pairs:
        key, value = pair.split("=")
        if key not in MAP_NAMES.keys():
            logger.warning(f"Key {key} is not a valid key in {MAP_NAMES.keys()}")
            raise SystemExit
        result[MAP_NAMES[key]] = value

    return result


def add_subcommand(parse):
    name = "config"
    subparser = parse.add_parser(name, help="Generate configuration file")
    subparser.add_argument(
        "set",
        metavar="key=value",
        nargs="*",
        action="extend",
        help="Set a number of key value pairs for configuration.",
    )

    return name


def cli(args):
    if args.set:
        default_path = Path.cwd() / CONFIG_FILE_NAME
        global_path = Path.home() / ".config" / CONFIG_FILE_NAME
        key_value_pairs = read_key_value_pair(args.set)

        if default_path.exists():
            config = toml.load(default_path)
            path = default_path
        elif global_path.exists():
            config = toml.load(global_path)
            path = global_path
        else:
            config = DEFAULT_CONFIG
            path = default_path

        for key, value in key_value_pairs.items():
            original_key = config["OpenAI"][key]
            if original_key == ["sk-key1", "sk-key2"]:
                config["OpenAI"][key] = [value]
            else:
                config["OpenAI"][key] = config["OpenAI"][key] + [value]

        create_config_file(config, path)

    else:
        create_config_file()
        logger.success("chatre.toml created")



import os
from pathlib import Path

import toml
from loguru import logger

CONFIG_FILE_NAME = "chatre.toml"


def report_token_usage(response):
    logger.info(f"prompt_token_used: {response.usage.prompt_tokens}")
    logger.info(f"completion_token_used: {response.usage.completion_tokens}")
    logger.info(f"total_token_used: {response.usage.total_tokens}")
    logger.info(f"response_time: { response.response_ms / 1000.0}s")


def load_config():
    default_path = Path.cwd() / CONFIG_FILE_NAME
    global_path = Path.home() / ".config" / CONFIG_FILE_NAME

    if default_path.exists():
        config = toml.load(default_path)
    elif global_path.exists():
        config = toml.load(global_path)
    else:
        raise FileNotFoundError("No apikey.toml found")

    chat_api_list = config["OpenAI"]["OPENAI_API_KEYS"]

    env_api = os.environ.get("OPENAI_API_KEY", None)
    if env_api is not None:
        chat_api_list.append(env_api)

    new_chat_api_list = []
    for i in chat_api_list:
        if len(i.strip()) < 51:
            logger.warning(f"API key {i} is too short, it will be skipped.")
        else:
            new_chat_api_list.append(i.strip())

    if len(new_chat_api_list) == 0:
        logger.error("No API key found")
        raise SystemExit

    return config, new_chat_api_list



import datetime
import os
from pathlib import Path

import arxiv
import tenacity
from loguru import logger
from pydantic import BaseModel, validator

from ..paper_with_image import Paper
from ..reader import BaseReader


class PaperParams(BaseModel):
    pdf_path: str
    query: str
    key_word: str
    filter_keys: str
    max_results: int
    sort: str
    save_image: bool
    file_format: str
    language: str

    @validator("pdf_path")
    def pdf_path_must_exist(cls, v):
        if not Path(v).exists():
            raise ValueError("pdf_path must exist")
        return v


# 定义Reader类
class Reader(BaseReader):
    # 初始化方法，设置属性
    def __init__(
        self,
        key_word,
        query,
        filter_keys,
        root_path=".",
        sort=arxiv.SortCriterion.SubmittedDate,
        user_name="defualt",
        args=None,
    ):
        if args is None:
            raise ValueError("args is None")

        if args.language == "en":
            language = "English"
        elif args.language == "zh":
            language = "Chinese"
        else:
            language = "English"

        super().__init__(
            filter_keys,
            root_path,
            language,
            args.file_format,
            args.save_image,
        )

        self.user_name = user_name  # 读者姓名
        self.key_word = key_word  # 读者感兴趣的关键词
        self.query = query  # 读者输入的搜索查询
        self.sort = sort  # 读者选择的排序方式

    def get_arxiv(self, max_results=30):
        search = arxiv.Search(
            query=self.query,
            max_results=max_results,
            sort_by=self.sort,
            sort_order=arxiv.SortOrder.Descending,
        )
        return search

    def filter_arxiv(self, max_results=30):
        search = self.get_arxiv(max_results=max_results)
        logger.info("all search:")
        for index, result in enumerate(search.results()):
            logger.info(f"{index=}, {result.title=}, {result.updated}")

        filter_results = []
        filter_keys = self.filter_keys

        logger.info(f"filter_keys {self.filter_keys}")
        # 确保每个关键词都能在摘要中找到，才算是目标论文
        for index, result in enumerate(search.results()):
            abs_text = result.summary.replace("-\n", "-").replace("\n", " ")
            meet_num = 0
            for f_key in filter_keys.split(" "):
                if f_key.lower() in abs_text.lower():
                    meet_num += 1
            if meet_num == len(filter_keys.split(" ")):
                filter_results.append(result)
                # break
        logger.info(f"filter_results: {len(filter_results)}")
        logger.info("filter_papers:")
        for index, result in enumerate(filter_results):
            logger.info(f"{index=}, {result.title=}, {result.updated}")
        return filter_results

    def download_pdf(self, filter_results):
        # 先创建文件夹
        date_str = str(datetime.datetime.now())[:13].replace(" ", "-")

        query_str = (
            self.query.replace("au:", "")
            .replace("title: ", "")
            .replace("ti: ", "")
            .replace(":", " ")[:25]
        )

        path = self.root_path / "pdf_files" / f"{query_str}-{date_str}"
        path.mkdir(parents=True, exist_ok=True)

        logger.info(f"All_paper: {len(filter_results)}")
        # 开始下载：
        paper_list = []

        for _, result in enumerate(filter_results):
            try:
                title_str = self.validateTitle(result.title)
                pdf_name = title_str + ".pdf"
                # result.download_pdf(path, filename=pdf_name)
                self.try_download_pdf(result, path.as_posix(), pdf_name)

                paper_path = path / pdf_name

                logger.info(f"{paper_path=}")

                paper = Paper(
                    path=paper_path,
                    url=result.entry_id,
                    title=result.title,
                    abs=result.summary.replace("-\n", "-").replace("\n", " "),
                    authers=[str(aut) for aut in result.authors],
                )

                paper_list.append(paper)
            except Exception as e:
                logger.warning(f"download_error: {e}")
                pass
        return paper_list

    @tenacity.retry(
        wait=tenacity.wait_exponential(multiplier=1, min=4, max=10),
        stop=tenacity.stop_after_attempt(5),
        reraise=True,
    )
    def try_download_pdf(self, result, path, pdf_name):
        result.download_pdf(path, filename=pdf_name)

    def show_info(self):
        logger.info(f"Key word: {self.key_word}")
        logger.info(f"Query: {self.query}")
        logger.info(f"Sort: {self.sort}")


def add_subcommand(parser):
    name = "paper"
    subparser = parser.add_parser(
        name, help="Fetch or Summary paper from local or arxiv"
    )
    subparser.add_argument(
        "--pdf-path",
        type=str,
        default="",
        metavar="",
        help="if none, the bot will download from arxiv with query",
    )

    subparser.add_argument(
        "--query",
        type=str,
        default="all: ChatGPT robot",
        metavar="",
        help="the query string, ti: xx, au: xx, all: xx (default: %(default)s)",
    )

    subparser.add_argument(
        "--key-word",
        type=str,
        default="reinforcement learning",
        metavar="",
        help="the key word of user research fields (default: %(default)s)",
    )

    subparser.add_argument(
        "--filter-keys",
        type=str,
        default="ChatGPT robot",
        metavar="",
        help="the filter key words, every word in the abstract must have, otherwise it will not be selected as the target paper (default: %(default)s)",
    )

    subparser.add_argument(
        "--max-results",
        type=int,
        default=1,
        metavar="",
        help="the maximum number of results (default: %(default)s)",
    )

    subparser.add_argument(
        "--sort",
        type=str,
        default="Relevance",
        metavar="",
        help="another is LastUpdatedDate (default: %(default)s)",
    )

    subparser.add_argument(
        "--save-image",
        default=False,
        metavar="",
        help="save image? It takes a minute or two to save a picture! But pretty (default: %(default)s)",
    )

    subparser.add_argument(
        "--file-format",
        type=str,
        default="md",
        choices=["md", "txt", "pdf"],
        metavar="",
        help="the format of the exported file, if you save the picture, it is best to be md, if not, the txt will not be messy (default: %(default)s)",
    )

    subparser.add_argument(
        "--language",
        type=str,
        default="en",
        metavar="",
        help="The other output lauguage is English, is en (default: %(default)s)",
    )

    return name


def main(args):
    # 创建一个Reader对象，并调用show_info方法
    if args.sort == "Relevance":
        sort = arxiv.SortCriterion.Relevance
    elif args.sort == "LastUpdatedDate":
        sort = arxiv.SortCriterion.LastUpdatedDate
    else:
        sort = arxiv.SortCriterion.Relevance

    if args.pdf_path:
        reader1 = Reader(
            key_word=args.key_word,
            query=args.query,
            filter_keys=args.filter_keys,
            sort=sort,
            args=args,
        )
        reader1.show_info()
        # 开始判断是路径还是文件：
        paper_list = []
        if args.pdf_path.endswith(".pdf"):
            paper_list.append(Paper(path=args.pdf_path))
            logger.info(f"read pdf file {args.pdf_path}")
        else:
            logger.info(f"read pdf files from path {args.pdf_path}")
            for root, dirs, files in os.walk(args.pdf_path):
                logger.trace(f"root: {root}, dirs: {dirs}, files: {files}")
                for filename in files:
                    # 如果找到PDF文件，则将其复制到目标文件夹中
                    if filename.endswith(".pdf"):
                        paper_list.append(Paper(path=os.path.join(root, filename)))

        logger.info("paper_num: {}".format(len(paper_list)))
        for paper_index, paper_name in enumerate(paper_list):
            name = paper_name.path.split("\\")[-1]
            logger.info(f"{paper_index=}, {name=}")

        reader1.summary_with_chat(paper_list=paper_list, key_words=reader1.key_word)
    else:
        reader1 = Reader(
            key_word=args.key_word,
            query=args.query,
            filter_keys=args.filter_keys,
            sort=sort,
            args=args,
        )
        reader1.show_info()
        filter_results = reader1.filter_arxiv(max_results=args.max_results)
        paper_list = reader1.download_pdf(filter_results)
        reader1.summary_with_chat(paper_list=paper_list, key_words=reader1.key_word)


def cli(args):
    parameters = PaperParams(**vars(args))
    main(parameters)



import datetime
import os
from pathlib import Path

import openai
import tenacity
import tiktoken
from loguru import logger
from pydantic import BaseModel, validator

from chat_research.utils import report_token_usage

from ..utils import load_config


# ChatResponse
class ResponseParams(BaseModel):
    comment_path: str
    file_format: str
    language: str

    @validator("comment_path")
    def comment_path_must_exist(cls, v):
        if not Path(v).exists():
            raise ValueError("comment_path must exist")
        return v


# 定义Response类
class Response:
    # 初始化方法，设置属性
    def __init__(self, args=None):
        if args is None:
            raise ValueError("args is None")

        if args.language == "en":
            self.language = "English"
        elif args.language == "zh":
            self.language = "Chinese"
        else:
            self.language = "Chinese"

        self.config, self.chat_api_list = load_config()

        self.cur_api = 0
        self.file_format = args.file_format
        self.max_token_num = 4096
        self.encoding = tiktoken.get_encoding("gpt2")

    def response_by_chatgpt(self, comment_path):
        htmls = []
        # 读取回复的内容
        with open(comment_path, "r", encoding="utf-8") as f:
            comments = f.read()

        chat_response_text = self.chat_response(text=comments)
        htmls.append(chat_response_text)

        # 将审稿意见保存起来
        date_str = str(datetime.datetime.now())[:13].replace(" ", "-")
        export_path = os.path.join("./", "response_file")
        if not os.path.exists(export_path):
            os.makedirs(export_path)
        file_name = os.path.join(
            export_path, date_str + "-Response." + self.file_format
        )
        self.export_to_markdown("\n".join(htmls), file_name=file_name)
        htmls = []

    @tenacity.retry(
        wait=tenacity.wait_exponential(multiplier=1, min=4, max=10),
        stop=tenacity.stop_after_attempt(5),
        reraise=True,
    )
    def chat_response(self, text):
        openai.api_key = self.chat_api_list[self.cur_api]
        self.cur_api += 1
        self.cur_api = (
            0 if self.cur_api >= len(self.chat_api_list) - 1 else self.cur_api
        )
        response_prompt_token = 1000
        text_token = len(self.encoding.encode(text))
        input_text_index = int(
            len(text) * (self.max_token_num - response_prompt_token) / text_token
        )
        input_text = "This is the review comments:" + text[:input_text_index]
        messages = [
            {
                "role": "system",
                "content": """You are the author, you submitted a paper, and the reviewers gave the review comments.
                Please reply with what we have done, not what we will do.
                You need to extract questions from the review comments one by one, and then respond point-to-point to the reviewers’ concerns.
                Please answer in {}. Follow the format of the output later:
                - Response to reviewers
                #1 reviewer
                Concern #1: xxxx
                Author response: xxxxx

                Concern #2: xxxx
                Author response: xxxxx
                ...

                #2 reviewer
                Concern #1: xxxx
                Author response: xxxxx

                Concern #2: xxxx
                Author response: xxxxx
                ...

                #3 reviewer
                Concern #1: xxxx
                Author response: xxxxx

                Concern #2: xxxx
                Author response: xxxxx
                ...

                """.format(
                    self.language
                ),
            },
            {"role": "user", "content": input_text},
        ]

        response = openai.ChatCompletion.create(
            model="gpt-3.5-turbo",
            messages=messages,
        )
        result = ""
        for choice in response.choices:
            result += choice.message.content
        logger.info("********" * 10)
        logger.info(result)
        logger.info("********" * 10)

        report_token_usage(response)

        return result

    def export_to_markdown(self, text, file_name, mode="w"):
        # 使用markdown模块的convert方法，将文本转换为html格式
        # html = markdown.markdown(text)
        # 打开一个文件，以写入模式
        with open(file_name, mode, encoding="utf-8") as f:
            # 将html格式的内容写入文件
            f.write(text)


def add_subcommand(parser):
    name = "response"
    subparser = parser.add_parser(name, help="Generate reponse for review comment")
    subparser.add_argument(
        "--comment-path",
        type=str,
        metavar="",
        help="path of comment",
        required=True,
    )
    subparser.add_argument(
        "--file-format",
        type=str,
        default="txt",
        metavar="",
        help="output file format (default: %(default)s)",
    )
    subparser.add_argument(
        "--language",
        type=str,
        default="en",
        metavar="",
        help="output language, en or zh (default: %(default)s)",
    )

    return name


def main(args):
    Response1 = Response(args=args)
    Response1.response_by_chatgpt(comment_path=args.comment_path)


def cli(args):
    parameter = ResponseParams(**vars(args))
    main(parameter)
